﻿using MemberLib;
using MemberManagerLib;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MemberManagerApp
{
    class MMApp
    {
        #region 단일체
        internal static MMApp Singleton
        {
            get;
            private set;
        }
        static MMApp()
        {
            Singleton = new MMApp();
        }
        MMApp()
        {
        }
        #endregion

        MemberManager mm = new MemberManager();

        internal void Run()
        {
            ConsoleKey key;
            while((key = SelectMenu())!=ConsoleKey.Escape)
            {
                switch (key)
                {
                    case ConsoleKey.F1: AddMember(); break;
                    case ConsoleKey.F2: RemoveMember(); break;
                    case ConsoleKey.F3: SearchMember(); break;
                    case ConsoleKey.F4: ViewAll(); break;
                    default: Console.WriteLine("잘못 선택하였습니다."); break;
                }
                Console.WriteLine("아무 키나 누르세요.");
                Console.ReadKey(true);
            }
        }

        private void ViewAll()
        {
            Console.WriteLine("*** F4: 전체 회원 데이터 보기");
            
            Console.WriteLine("*** F1:상세 보기 그 외: 간략 보기");
            if(Console.ReadKey(true).Key == ConsoleKey.F1)
            {
                mm.DoItAll(VerifyView);
            }
            else
            {
                mm.DoItAll(SimplyView);
            }
        }
        static void VerifyView(Member member)
        {
            Console.WriteLine("아이디:{0}", member.ID);
            Console.WriteLine("*  이름:{0}", member.Name);
            Console.WriteLine("*  주소:{0}", member.Address);
            Console.WriteLine("*  나이:{0}", member.Age);
        }
        static void SimplyView(Member member)
        {
            Console.WriteLine("<{0}:{1}>", member.ID, member.Name);
        }

        string dummy = string.Empty;//검색을 위해 필요한 더미 멤버
        private void SearchMember()
        {
            Console.WriteLine("*** F3: 회원 데이터 검색");

            Console.WriteLine("*** F1:ID로 검색, 그 외: 이름으로 검색");


            List<Member> members;
            if (Console.ReadKey(true).Key == ConsoleKey.F1)
            {
                Console.Write("검색할 회원 ID:");
                dummy = Console.ReadLine();
                members = mm.SearchMember(CompareByID);
            }
            else
            {
                Console.Write("검색할 회원 이름:");
                dummy = Console.ReadLine();
                members = mm.SearchMember(CompareByName);
            }

            Console.WriteLine("=== 검색 결과");
            if(members.Count == 0)
            {
                Console.WriteLine("조건에 맞는 회원 데이터는 없네요.");
                return;
            }
            foreach (Member member in members)
            {
                VerifyView(member);
            }
        }
        bool CompareByID(Member member)
        {
            return member.ID == dummy;
        }
        bool CompareByName(Member member)
        {
            return member.Name == dummy;
        }

        private void RemoveMember()
        {
            Console.WriteLine("*** F2: 회원 데이터 삭제");
            Console.Write("삭제할 회원 ID:");
            string id = Console.ReadLine();
            if(mm.RemoveMember(id))
            {
                Console.WriteLine("삭제하였습니다.");
            }
            else
            {
                Console.WriteLine("회원 데이터 삭제 실패!!!");
                Console.WriteLine("존재하는 회원 ID인지 확인해 보세요.");
            }
        }

        private void AddMember()
        {
            Console.WriteLine("*** F1: 회원 데이터 추가");
            Console.Write("추가할 회원 ID:");
            string id = Console.ReadLine();
            Console.Write("이름:");
            string name = Console.ReadLine();
            Console.Write("주소:");
            string addr = Console.ReadLine();
            Console.Write("나이:");
            int age = -1;
            int.TryParse(Console.ReadLine(), out age);

            if(mm.AddMember(id,name,addr,age))
            {
                Console.WriteLine("추가하였습니다.");
            }
            else
            {
                Console.WriteLine("회원 데이터 추가 실패!!!");
                Console.WriteLine("이미 존재하는 아이디인지 확인하세요.");
            }

        }

        private ConsoleKey SelectMenu()
        {
            Console.Clear();
            Console.WriteLine("*** 회원 관리 프로그램");
            Console.WriteLine("*** F1: 회원 데이터 추가");
            Console.WriteLine("*** F2: 회원 데이터 삭제");
            Console.WriteLine("*** F3: 회원 데이터 검색");
            Console.WriteLine("*** F4: 전체 회원 데이터 보기");
            return Console.ReadKey(true).Key;
        }
    }
}
