﻿using MemberLib;
using System.Collections.Generic;

namespace MemberManagerLib
{
    /// <summary>
    /// 회원 관리자 클래스
    /// </summary>
    public class MemberManager
    {
        Dictionary<string, Member> mdic = new Dictionary<string, Member>();
        /// <summary>
        /// 회원 데이터 추가 메서드
        /// </summary>
        /// <param name="id">아이디</param>
        /// <param name="name">이름</param>
        /// <param name="addr">주소</param>
        /// <param name="age">나이</param>
        /// <returns>추가 여부</returns>
        public bool AddMember(string id, string name,string addr,int age)
        {
            if(mdic.ContainsKey(id))
            {
                return false;
            }
            mdic[id] = new Member(id, name, addr, age);
            return true;
        }

        /// <summary>
        /// 회원 데이터 삭제 메서드
        /// </summary>
        /// <param name="id">아이디</param>
        /// <returns>삭제 여부</returns>
        public bool RemoveMember(string id)
        {
            if(mdic.ContainsKey(id))
            {
                mdic.Remove(id);
                return true;
            }
            return false;
        }
        /// <summary>
        /// 회원 데이터 검색 메서드
        /// </summary>
        /// <param name="cdele">비교 대리자</param>
        /// <returns>조건에 만족하는 회원 컬렉션</returns>
        public List<Member> SearchMember(CompareDele cdele)
        {
            List<Member> members = new List<Member>();
            foreach(Member member in mdic.Values)
            {
                if(cdele(member))
                {
                    members.Add(member);                    
                }
            }
            return members;
        }
        /// <summary>
        /// 모든 멤버에게 특정 행위를 수행하는 메서드
        /// </summary>
        /// <param name="doitmember">수행할 메서드의 대리자</param>
        public void DoItAll(DoItMember doitmember)
        {
            foreach(Member member in mdic.Values)
            {
                doitmember(member);
            }
        }
    }
}
